package net.onthewings.utils {
	import flash.utils.ByteArray;

	public class ColorUtil {
		/**
		 * Convert a RBG(in hex) color to HSL color space.
		 * Modified from wikipedia http://en.wikipedia.org/wiki/HSL_and_HSV and mojocolor http://code.google.com/p/mojocolors/
		 *
		 * @param color The source color in hex. eg:0xFF0000
		 * @return A array in [h,s,l] format. h: [0,360], s: [0,1], l: [0,1].
		 */
		public function convertRGBToHSL(color:uint):Array {
			var rgb:Array = [];
			var min:Number, max:Number, delta:Number, h:Number, s:Number, l:Number;

			rgb[0] = color >> 16 & 0xFF;
			rgb[1] = color >> 8 & 0xFF;
			rgb[2] = color & 0xFF;
			rgb[0] /= 255;
			rgb[1] /= 255;
			rgb[2] /= 255;

			min = Math.min(rgb[0], rgb[1], rgb[2]);
			max = Math.max(rgb[0], rgb[1], rgb[2]);

			l = 0.5 * (max + min);
			delta = max - min;

			if (max != 0.0) {
				if (rgb[0] == max) {
					h = (rgb[1] - rgb[2]) / delta; // between yellow & magenta
				} else if (rgb[1] == max) {
					h = 2.0 + (rgb[2] - rgb[0]) / delta; // between cyan & yellow
				} else {
					h = 4.0 + (rgb[0] - rgb[1]) / delta; // between magenta & cyan
				}
				h *= 60.0;
				// degrees
				if (h < 0.0) {
					h += 360.0;
				}
			} else {
				h = -1.0;
			}

			if (l == 0.0) {
				s = 0.0;
			} else if (l <= 0.5) {
				s = delta / (2.0 * l);
			} else { //l > 0.5
				s = delta / (2.0 - 2.0 * l);
			}

			return [h, s, l];
		}

		/**
		 * Convert a RBG(in hex) color to HSV color space.
		 * Modified from mojocolor http://code.google.com/p/mojocolors/
		 *
		 * @param color The source color in hex. eg:0xFF0000
		 * @return A array in [h,s,v] format. h: [0,360], s: [0,1], v: [0,1].
		 */
		public function convertRGBToHSV(color:uint):Array {
			var red:Number = (color >> 16 & 0xFF) / 255;
			var green:Number = (color >> 8 & 0xFF) / 255;
			var blue:Number = (color & 0xFF) / 255;
			var min:Number, max:Number, delta:Number;

			min = Math.min(red, green, blue);
			max = Math.max(red, green, blue);
			var v:Number = max;
			// v
			var h:Number;
			delta = max - min;
			var s:Number;
			if (max != 0) {
				s = delta / max; // s
			} else {
				// r = g = b = 0		// s = 0, v is undefined
				s = 0;
				h = -1;
				return new Array(h, s * 100, v);
			}

			if (red == max) {
				h = (green - blue) / delta; // between yellow & magenta
			} else if (green == max) {

				h = 2 + (blue - red) / delta; // between cyan & yellow
			} else {
				h = 4 + (red - green) / delta; // between magenta & cyan
			}
			h *= 60;
			// degrees
			if (h < 0) {
				h += 360;
			}
			return [h, s, v];
		}
	}
}